function segregation_EN

%% Script to:
% -individual subject-level parcellation
% -group-level parcellation
% Please open the script and enter the required variables
%==========================================================================
%	Copyright  2016
%   NINET Lab, Directed by Prof. Fidel Vila-Rodriguez
%	University of British Columbia
%	Written by Ruiyang Ge, Ph.D
%	Mail to Author:  <a href="ruiyang.ge@ubc.ca">Ruiyang Ge</a>
%	Mail to Author:  <a href="ninet.lab@ubc.ca">NINET Lab</a>
%   Version 1.0;
%   Date 20160831;
%==========================================================================
%   Reference: Ruiyang Ge, Adam A. Dipinto, Daniel Blumberger, Jonathan Downar,...
%              Zafiris Daskalakis, Joe Tham, Raymond Lam, Fidel Vila-Rodriguez,... 
%              A Sparse Representation-based Method for Parcellation of the Resting Brain...
%              and Its Application to Treatment-resistant Major Depressive
%              Disorder, In preparation.
%==========================================================================
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%==========================================================================


% NOTE THAT THE SPM(SPM8 or higher version) MATLAB DIRECTORY MUST BE INSTALLED,
%
% ---------------------------------------------------------------

clear;clc;
OriginalPath = pwd;

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scrsz = get(0,'ScreenSize');

[PathName_CNR] = uigetdir('D:\','Select the File Folder');
FileList_CNR = dir(PathName_CNR);
PathName = [PathName_CNR];
FileList = dir(PathName);

PathName_CNR2 = uigetdir('D:\','Select the Save Path');
[PathName2] = [PathName_CNR2];

numSub = length(FileList_CNR)-2;

%% mask
    [MASK_name,MASK_path] = uigetfile('*.img;*.nii','Select the target mask');
    MASK = [MASK_path,MASK_name];
    V_mask = spm_vol(MASK);
    Data_mask = spm_read_vols(V_mask);  
    index = find(Data_mask~=0);
    

    lambda = [1.0]';%%% this value controls the overall strength of the penalty (including both L-1 and L-2) with a range of lamda>=0 
    alpha = [0.7]'; %%% this value compromise parameter between the L-1 and L-2 constraints with a range of [0,1]
    
    %%% enter the number(s) of parcels; the users could enter multiple
    %%% numbers as they want to parcellate the target area
    input_kk = inputdlg('Please input the numbers of parcels(numbers should be segrated by spaces!)');
    if ~isempty(round(str2num(input_kk{1})))
        kk = round(str2num(input_kk{1}));
    else
        error('The input should be integer(s)');
    end
    
    for i = 1:length(kk)
        new_folder = [PathName2 '\K=',sprintf('%01d',kk(i))];
        mkdir(new_folder);
    end
    
    %%% enter the name of the target area
    input_name = inputdlg('Please input the name of the target area (avoid spaces!)');
    target_name = input_name{1};
    
for K_index = 1:length(kk)
    
    K = kk(K_index);%%% K is the number of pacels
    
  for lam = length(lambda):-1:1
      for alp = length(alpha):-1:1
        
    hh = waitbar(1,'computing...','Position',[scrsz(4)/2 scrsz(4)/2 scrsz(3)/5 scrsz(4)/15]);
    IDX_all = zeros(length(index),1,numSub);
    Centroid_all = zeros(K,length(index),numSub);
    CM_all = zeros(length(index),length(index),numSub);
  
%% individual-level parcellation    
for ijk = 1:numSub
    
    waitbar(ijk/numSub,hh,sprintf('# subject %03d',ijk));
    
    DataDir = [PathName,'\',sprintf(FileList_CNR(ijk+2,1).name)];%%% data path
    
    SaveDir = [PathName2 '\K=',sprintf('%01d',K)];%%% path for results
    
    [Data, Header]  = read_files_gry(DataDir,OriginalPath);%%% read the original data
            
    volumesize = size(Data,1)*size(Data,2)*size(Data,3);%%% volume size of the original data
    if volumesize ~= size(Data_mask,1)*size(Data_mask,2)*size(Data_mask,3)
        close(hh);
        errordlg('The size of data is not identical with that of the ROI region');
    end
    
    
    %%% mask
    Data_MASK = repmat(Data_mask,[1,1,1,size(Data,4)]);

    %%% reshape the masked data
    Data_Masked = Data.*Data_MASK;
    Data_Masked_reshape = reshape(Data_Masked,volumesize,size(Data,4));
    
    %%% retrieve the time-courses within the selected ROI
    timecourses = Data_Masked_reshape(index,:)';
       
    %%% elastic net
    coef_matrix = zeros(size(timecourses,2),size(timecourses,2));
    for i = 1:size(timecourses,2)
        
        seed = timecourses(:,i);
        residual_matrix = timecourses(:,[1:i-1, i+1:end]);         
    %%% GLMNET
        options = glmnetSet;
        options.alpha = alpha(alp,1);
        options.lambda = lambda(lam,1);
        fit = glmnet(residual_matrix,seed,[],options);
        representation_coef = fit.beta;      
              
        representation_coefs = [representation_coef(1:i-1);0;representation_coef(i:end)];
        coef_matrix(i,:) = representation_coefs';
    end
    
    SM = coef_matrix*coef_matrix';

    %%% spectral clustering 
    affinity = abs(SM);
    [IDX, L, U] = SpectralClustering(affinity, K, 3);

    %%% remap the clustered data for display
    Data_mask_reshape = reshape(Data_mask,1,volumesize);
    Data_mask_reshape(1,index) = IDX;
    Data_remapped = reshape(Data_mask_reshape,size(Data,1),size(Data,2),size(Data,3));
    
    name = [target_name,'_',sprintf(FileList_CNR(ijk+2,1).name)];
    write_files_gry(Data_remapped,Header,SaveDir,OriginalPath,name);
    
    %%% remap the clustered data for each cluster
    for mn = 1:K
        Data_mask_reshape_each  = (Data_mask_reshape==mn);
        Data_remapped_each = reshape(Data_mask_reshape_each,size(Data,1),size(Data,2),size(Data,3));
        name_each = [target_name,'_',sprintf(FileList_CNR(ijk+2,1).name),'_',sprintf(num2str(mn)),'seed'];
        write_files_gry(Data_remapped_each,Header,SaveDir,OriginalPath,name_each);
    end


    cd(SaveDir);
    IDX_all(:,:,ijk) = IDX;   %%% IDX_all is the index labels for all individual parcellations
end
     IDX_all_name = ['IDX_all'];
     save(IDX_all_name,'IDX_all');

     fprintf('\n Spectral Clustering computation at single-subject-level Done.\n');
     close(hh);cd(OriginalPath);

%% group-level parcellation
     cd(SaveDir);
     numVoxels = size(index,1);

     %%% consensus matrix for each subject
     for ij = 1:numSub
         IDX = IDX_all(:,:,ij);
         As = zeros(numVoxels); %%% adjacency matrix
         for i=1:numVoxels
             for j=1:numVoxels
                 if IDX(i)==IDX(j)
                    As(i,j) = 1;      
                 end
             end
         end
         AS(:,:,ij) = As;
      
     end

     AS_name = [target_name,'_AS'];
     save(AS_name,'AS');

    %%% group-level consensus matrix A
    A_brainArea = mean(AS,3);
    
    %%% spcetral clustering
    [IDX_group] = SpectralClustering(A_brainArea, K, 3); %%% IDX_group is the index labels for the group parcellation
    IDX_group_name = ['IDX_group'];
    save(IDX_group_name,'IDX_group');

    %%% remap the group clustered data for display
    Data_mask_reshape_group = reshape(Data_mask,1,volumesize);
    Data_mask_reshape_group(1,index) = IDX_group;
    Data_remapped_group = reshape(Data_mask_reshape_group,size(Data,1),size(Data,2),size(Data,3));
    name_group = [target_name,'_group'];
    write_files_gry(Data_remapped_group,Header,SaveDir,OriginalPath,name_group);
    
    %%% remap the clustered data for each cluster
    for mn = 1:K
        Data_mask_reshape_each  = (Data_mask_reshape_group==mn);
        Data_remapped_each = reshape(Data_mask_reshape_each,size(Data,1),size(Data,2),size(Data,3));
        name_each = [target_name,'_group_',sprintf(num2str(mn)),'seed'];
        write_files_gry(Data_remapped_each,Header,SaveDir,OriginalPath,name_each);
    end
    
    
    fprintf('\n Spectral Clustering computation at group-level Done.\n\n\n\n');

    %%% go back to original path
    cd(OriginalPath);

    end
    end
end
% end
    
    
    






function write_files_gry(AllVolume,V,SaveDir,OriginalPath,fname)

% clear;clc;

% DataDir = 'D:\Documents\ICA\FastICA_code_SimTB\data_run\real data\Newark\data\sub01';
fprintf('\n Dictionary of the saved file: "%s".', SaveDir);
fprintf('\n Writing file...');
cd(SaveDir);
fname_new = [SaveDir,'\',fname,'.img'];
V.fname = fname_new;
spm_write_vol(V,AllVolume);
fprintf('\n');
cd(OriginalPath);




function [AllVolume, Header] = read_files_gry(DataDir,OriginalPath)

% clear;clc;

% DataDir = 'D:\Documents\ICA\FastICA_code_SimTB\data_run\real data\Newark\data\sub01';
fprintf('\n Dictionary of the files: "%s".', DataDir);
FileList = dir(DataDir);	
ImgFileList ={};
if length(FileList)==1
    h = waitbar(0,'Reading functional data...');
    V = spm_vol(DataDir);
    AllVolume = spm_read_vols(V); 
    VV = V(end,:);
    Header = VV;
    Header.n = [1 1];
else
	for x = 3:size(struct2cell(FileList),2)
	    if strcmpi(FileList(x).name(end-3:end), '.hdr') 
	        if strcmpi(FileList(x).name(1:end-4), FileList(x+1).name(1:end-4))
				ImgFileList=[ImgFileList; {FileList(x).name(1:end-4)}];
            else
                error('*.hdr & *.img files should be pairwise.');
            end
        elseif strcmpi(FileList(x).name(end-3:end), '.nii') 
            ImgFileList=[ImgFileList; {FileList(x).name(1:end-4)}];
	    end
	end

fprintf('\n Read functional data: "%s".', DataDir);

h = waitbar(0,'Reading functional data...');
for i = 1:size(ImgFileList,1)    		
		waitbar(i/size(ImgFileList,1),h)
   
        if strcmpi(FileList(x).name(end-3:end), '.img') 
            Filename = [DataDir,'\',ImgFileList{i},'.img'];        	
            Filename = deblank(Filename);
            V = spm_vol(Filename);
            Data = spm_read_vols(V);   
            Data(isnan(Data)) = 0;
            Header = V;
            AllVolume(:,:,:,i) = Data;
        else
            Filename = [DataDir,'\',ImgFileList{i},'.nii'];
            Filename = deblank(Filename);
            V = spm_vol(Filename);
            Data = spm_read_vols(V);   
            Data(isnan(Data)) = 0;
            VV = V(end,:);
            Header = VV;
            Header.n = [1 1];
            AllVolume = Data;
        end	
        
end  
end
close(h);
% fprintf('\n');
cd(OriginalPath);